import { Loggly } from 'winston-loggly-bulk'
import winston = require("winston");

export type LogLevel = "debug" | "info" | "warn" | "error"

type LogCallback = (error?: any, level?: string | undefined, message?: string | undefined, meta?: any) => void

interface LogAdapter {
    log: (level: string, msg: string, details: any, cb: LogCallback) => void
}

export interface LogOptions {
    adapter: LogAdapter
    details: any
    level: LogLevel
}


var logger: LogOptions = {
    adapter: winston,
    details: {},
    level: "debug"
}

winston.addColors({
    silly: 'magenta',
    debug: 'blue',
    verbose: 'cyan',
    info: 'green',
    warn: 'yellow',
    error: 'red'
});

const console = new winston.transports.Console({
    level: logger.level,
    format: winston.format.combine(
        winston.format.prettyPrint({ colorize: true, }),
    )
})
winston.remove(winston.transports.Console);
winston.add(console)

export const enableLoggly = (apiKey: string, identifier: string, env: string) => {
    winston.add(new Loggly({
        inputToken: apiKey,
        subdomain: 'quikserve',
        tags: [env, identifier],
        json: true
    }))
}



export const setOptions = (opts: LogOptions) => {
    logger = {
        ...logger,
        ...opts
    }
}

export function log(level: LogLevel, msg: string, details: any): Promise<void> {
    return new Promise((resolve) => {
        logger.adapter.log(level, msg, { ...details, ...logger.details }, resolve)
    })
}

export const debug = async (msg: string, details: any = {}) => log("debug", msg, details)
export const info = async (msg: string, details: any = {}) => log("info", msg, details)
export const warn = async (msg: string, details: any = {}) => log("warn", msg, details)
export const error = async (msg: string, details: any = {}) => log("error", msg, details)
